/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/09/14 16:49:15
 */

import { css, raw, html, Component, nextTick, styleMap } from 'wkit'
import ICONS from './svg.js'
import '../form/input.js'
import '../form/button.js'

const ACTTION = {
  bold: 'bold',
  italic: 'italic',
  under: 'underline',
  delete: 'strikeThrough',
  left: 'justifyLeft',
  center: 'justifyCenter',
  right: 'justifyRight',
  image: 'insertImage',
  font: 'fontSize',
  color: 'foreColor',
  link: 'createLink',
  ordered: 'insertOrderedList',
  unordered: 'insertUnorderedList'
}

const DEFAULT_TOOLS = [
  'font',
  'color',
  'bold',
  'italic',
  'under',
  'delete',
  'ordered',
  'unordered',
  'left',
  'center',
  'right',
  'link',
  'image'
]

const COLORS = [
  '#f3f5fb',
  '#dae1e9',
  '#62778d',
  '#58d68d',
  '#3fc2a7',
  '#52a3de',
  '#ac61ce',
  '#ffb618',
  '#e67e22',
  '#ff5061',
  '#ff0000',
  '#000000'
]

class Editor extends Component {
  static props = {
    toolbar: {
      type: String,
      default: '',
      attribute: false,
      observer(v) {
        if (v === null) {
          this.#toolbar = [...DEFAULT_TOOLS]
        } else if (v) {
          this.#toolbar = v.split(',').map(it => it.trim())
        }
      }
    },
    value: 'str!',
    readonly: false,
    disabled: false
  }

  static styles = [
    css`
      :host {
        display: flex;
        min-width: 200px;
        min-height: 100px;
        max-height: 720px;
        border-radius: 3px;
        transition: box-shadow 0.15s linear;
      }
      table {
        width: 100%;
        border-spacing: 0;
        border-collapse: collapse;

        tr {
          background: #fff;
        }
        thead tr {
          background: var(--color-plain-1);
        }
        th,
        td {
          padding: 6px 13px;
          border: 1px solid var(--color-plain-2);
          vertical-align: middle;
        }
        th {
          font-weight: bold;
        }
        tr:nth-child(2n) {
          background: #fcfdff;
        }
      }

      ul,
      ol {
        margin-left: 1em;
      }
      a {
        color: var(--color-teal-1);
      }
      .noselect {
        -webkit-touch-callout: none;
        -webkit-user-select: none;
        -moz-user-select: none;
        user-select: none;
      }
    `,

    css`
      .editor {
        position: relative;
        flex: 1;
        display: flex;
        flex-direction: column;
        width: 100%;
        border: 1px solid #e7e8eb;
        border-radius: inherit;
        font-size: 14px;
      }
      .toolbar {
        display: flex;
        height: 34px;
        padding: 5px;
        line-height: 24px;
        border-bottom: 1px solid #e7e8eb;

        span {
          position: relative;
          overflow: hidden;
          display: flex;
          justify-content: center;
          align-items: center;
          width: 24px;
          height: 24px;
          margin: 0 3px;
          border-radius: 3px;

          input {
            position: absolute;
            width: 100%;
            height: 100%;
            opacity: 0;
          }
          .icon {
            overflow: hidden;
            width: 70%;
            height: 70%;
            fill: currentColor;
            color: #62778d;
          }

          &:hover {
            background: #f7f8fb;
          }
        }
      }

      .scroll-outerbox {
        overflow: hidden;
        position: relative;
        flex: 1;
        padding: 5px 8px;
      }
      .scroll-innerbox {
        overflow-y: auto;
        width: 100%;
        height: 100%;
        scrollbar-width: 0;

        &::-webkit-scrollbar {
          display: none;
        }
      }

      .is-vertical {
        // visibility: hidden;
        position: absolute;
        right: 0;
        top: 0;
        display: flex;
        justify-content: flex-end;
        width: 10px;
        height: 100%;
        // opacity: 0;
        transition: opacity 0.3s linear, visibility 0.3s linear;

        .thumb {
          display: block;
          width: 6px;
          height: 90px;
          border-radius: 5px;
          background: rgba(44, 47, 53, 0.25);
          cursor: default;
          transition: width 0.1s linear, height 0.1s linear;

          &:hover {
            width: 10px;
            background: rgba(44, 47, 53, 0.5);
          }
        }
      }

      .typearea {
        width: 100%;
        height: 100%;
        outline: none;
        text-wrap: wrap;
        word-break: break-all;

        img {
          max-width: 100%;
        }
      }
    `,

    css`
      :host(:hover) {
        .is-vertical {
          visibility: visible;
          opacity: 1;
        }
      }
      :host([disabled]) {
        .editor {
          cursor: not-allowed;
          opacity: 0.6;
        }
      }

      :host([readonly]) {
        .editor {
          cursor: default;
          opacity: 0.8;
        }
      }
      :host([readonly]),
      :host([disabled]) {
        .toolbar {
          span:hover {
            background: none;
          }
        }
      }

      :host(:focus-within) {
        box-shadow: 0 0 0 2px var(--color-plain-a);
      }
    `,

    css`
      @use 'sass:list';

      .font-layer,
      .color-layer,
      .link-layer {
        visibility: hidden;
        position: absolute;
        left: 0;
        top: 0;
        z-index: 99;
        width: 80px;
        padding: 5px 0;
        line-height: 25px;
        background: #fff;
        box-shadow: 0 0 8px rgba(0, 0, 0, 0.2);
        font-size: 13px;
        opacity: 0;
        transition: all ease-in-out 0.2s;

        &.fadein {
          visibility: visible;
          top: 34px;
          opacity: 1;
        }
      }

      .font-layer {
        span {
          display: block;
          padding: 0 8px;

          &:hover {
            background: #f7f8fb;
          }
        }
      }
      $colors: #f3f5fb, #dae1e9, #62778d, #58d68d, #3fc2a7, #52a3de, #ac61ce,
        #ffb618, #e67e22, #ff5061, #ff0000, #000000;

      .color-layer {
        display: flex;
        flex-flow: row wrap;
        justify-content: center;
        left: 30px;
        width: 100px;

        span {
          width: 20px;
          height: 20px;
          margin: 2px;

          @for $i from 1 to 13 {
            &:nth-child(#{$i}) {
              background: list.nth($colors, $i);
            }
          }
        }
      }

      .link-layer {
        display: flex;
        flex-direction: column;
        left: 330px;
        width: 230px;
        padding: 8px;

        wc-button {
          width: 40px;
          margin-top: 8px;
        }
      }
    `
  ]

  #toolbar = []

  render() {
    toolbar = [
      ...(this.#toolbar.length ? this.#toolbar : DEFAULT_TOOLS),
      'copy'
    ]

    return html`
      <div class="editor">
        <section class="toolbar">
          ${toolbar.map(
            it => html`
              <span data-act=${it}>
                <svg class="icon" viewBox="0 0 1024 1024">
                  <path d=${ICONS[it]} />
                </svg>
                ${it === 'image'
                  ? html`<input type="file" accept="image/*" />`
                  : ''}
              </span>
            `
          )}
        </section>
        <div class="scroll-outerbox">
          <div class="scroll-innerbox">
            <div contenteditable="true" class="typearea" spellcheck="false">
              ${raw(this.value)}
            </div>
          </div>
          <div class="is-vertical noselect">
            <span ref="y" class="thumb"></span>
          </div>
        </div>
        <div class="font-layer noselect">
          <span data-size="6">6号字体</span>
          <span data-size="5">5号字体</span>
          <span data-size="4">4号字体</span>
          <span data-size="3">3号字体</span>
          <span data-size="2">2号字体</span>
        </div>
        <div class="color-layer noselect">
          ${COLORS.map(c => html`<span data-value=${c}></span>`)}
        </div>
        <div class="link-layer noselect">
          <wc-input label="请输入链接地址"></wc-input>
          <wc-button color="teal" size="mini">插入</wc-button>
        </div>
      </div>
    `
  }
}

Editor.reg('editor')
